@@ -6,12 +6,16 @@ log = logging.getLogger(__name__)
 
 
 class Bitbucket(AtlassianRestAPI):
-    def project_list(self):
+    def project_list(self, limit=None):
         """
         Provide the project list
+        :param limit: OPTIONAL 25 is default
         :return:
         """
-        return (self.get('rest/api/1.0/projects') or {}).get('values')
+        params = {}
+        if limit:
+            params['limit'] = limit
+        return (self.get('rest/api/1.0/projects', params=params) or {}).get('values')
 
     def project(self, key):
         """
@@ -22,6 +26,21 @@ class Bitbucket(AtlassianRestAPI):
         url = 'rest/api/1.0/projects/{0}'.format(key)
         return (self.get(url) or {}).get('values')
 
+    def create_project(self, key, name, description=""):
+        """
+        Create project
+        :param key:
+        :param name:
+        :param description:
+        :return:
+        """
+        url = 'rest/api/1.0/projects'
+        data = {"key": key,
+                "name": name,
+                "description": description
+                }
+        return self.post(url, data=data)
+
     def project_users(self, key, limit=99999):
         """
         Get users who has permission in project
@@ -129,6 +148,58 @@ class Bitbucket(AtlassianRestAPI):
             log.info('Next page start at {}'.format(response.get('nextPageStart')))
         return (response or {}).get('values')
 
+    def create_repo(self, project_key, repository, forkable=False, is_private=True):
+        """Create a new repository.
+
+        Requires an existing project in which this repository will be created. The only parameters which will be used
+        are name and scmId.
+
+        The authenticated user must have PROJECT_ADMIN permission for the context project to call this resource.
+
+        :param project_key: The project matching the projectKey supplied in the resource path as shown in URL.
+        :type project_key: str
+        :param repository: Name of repository to create (i.e. "My repo").
+        :type repository: str
+        :param forkable: Set the repository to be forkable or not.
+        :type forkable: bool
+        :param is_private: Set the repository to be private or not.
+        :type is_private: bool
+        :return:
+            201 - application/json (repository)
+            400 - application/json (errors)
+            401 - application/json (errors)
+            409 - application/json (errors)
+        :rtype: requests.Response
+        """
+
+        url = 'rest/api/1.0/projects/{projectKey}/repos'.format(projectKey=project_key)
+        data = {
+            "name": repository,
+            "scmId": "git",
+            "forkable": forkable,
+            "is_private": is_private
+        }
+        return self.post(url, data=data)
+
+    def repo_all_list(self, project_key):
+        """
+        Get all repositories list from project
+        :param project_key:
+        :return:
+        """
+        url = 'rest/api/1.0/projects/{projectKey}/repos'.format(projectKey=project_key)
+        params = {}
+        start = 0
+        params['start'] = start
+        response = self.get(url, params=params)
+        repo_list = (response or {}).get('values')
+        while not response.get('isLastPage'):
+            start = response.get('nextPageStart')
+            params['start'] = start
+            response = self.get(url, params=params)
+            repo_list += (response or {}).get('values')
+        return repo_list
+
     def get_branches(self, project, repository, base=None, filter=None, start=0, limit=99999, details=True,
                      order_by='MODIFICATION'):
         """
@@ -162,6 +233,37 @@ class Bitbucket(AtlassianRestAPI):
 
         return (self.get(url, params=params) or {}).get('values')
 
+    def create_branch(self, project_key, repository, name, start_point, message=""):
+        """Creates a branch using the information provided in the request.
+
+        The authenticated user must have REPO_WRITE permission for the context repository to call this resource.
+
+        :param project_key: The project matching the projectKey supplied in the resource path as shown in URL.
+        :type project_key: str
+        :param repository: Name of repository where branch is created (i.e. "my_repo").
+        :type repository: str
+        :param name: Name of branch to create (i.e. "my_branch").
+        :type name: str
+        :param start_point: Name of branch to branch from.
+        :type start_point: str
+        :param message: Branch message.
+        :type message: str
+        :return:
+            200 - application/json (repository)
+            401 - application/json (errors)
+            404 - application/json (errors)
+        :rtype: requests.Response
+        """
+
+        url = 'rest/api/1.0/projects/{projectKey}/repos/{repository}/branches'.format(projectKey=project_key,
+                                                                                      repository=repository)
+        data = {
+            "name": name,
+            "startPoint": start_point,
+            "message": message
+        }
+        return self.post(url, data=data)
+
     def delete_branch(self, project, repository, name, end_point):
         """
         Delete branch from related repo
@@ -562,3 +664,66 @@ class Bitbucket(AtlassianRestAPI):
             # check as support tools
             response = self.get('rest/supportHealthCheck/1.0/check/')
         return response
+
+    def get_branching_model(self, project, repository):
+        """
+        Get branching model
+        :param project:
+        :param repository:
+        :return:
+        """
+        url = 'rest/branch-utils/1.0/projects/{project}/repos/{repository}/branchmodel/configuration'.format(project=project,
+                                                                                                             repository=repository)
+        return self.get(url)
+
+    def set_branching_model(self, project, repository, data):
+        """
+        Set branching model
+        :param project:
+        :param repository:
+        :param data:
+        :return:
+        """
+        url = 'rest/branch-utils/1.0/projects/{project}/repos/{repository}/branchmodel/configuration'.format(project=project,
+                                                                                                             repository=repository)
+        return self.put(url, data=data)
+
+    def enable_branching_model(self, project, repository):
+        """
+        Enable branching model by setting it with default configuration
+        :param project:
+        :param repository:
+        :return:
+        """
+        default_model_data = {'development': {'refId': None, 'useDefault': True},
+                              'types': [{'displayName': 'Bugfix',
+                                         'enabled': True,
+                                         'id': 'BUGFIX',
+                                         'prefix': 'bugfix/'},
+                                        {'displayName': 'Feature',
+                                         'enabled': True,
+                                         'id': 'FEATURE',
+                                         'prefix': 'feature/'},
+                                        {'displayName': 'Hotfix',
+                                         'enabled': True,
+                                         'id': 'HOTFIX',
+                                         'prefix': 'hotfix/'},
+                                        {'displayName': 'Release',
+                                         'enabled': True,
+                                         'id': 'RELEASE',
+                                         'prefix': 'release/'}]}
+        return self.set_branching_model(project,
+                                        repository,
+                                        default_model_data)
+
+    def disable_branching_model(self, project, repository):
+        """
+        Disable branching model
+        :param project:
+        :param repository:
+        :param data:
+        :return:
+        """
+        url = 'rest/branch-utils/1.0/projects/{project}/repos/{repository}/branchmodel/configuration'.format(project=project,
+                                                                                                             repository=repository)
+        return self.delete(url)
