@@ -53,6 +53,14 @@ class Confluence(AtlassianRestAPI):
             log.error(e)
             return None
 
+    def get_child_pages(self, page_id):
+        """
+        Get child pages for the provided page_id
+        :param page_id:
+        :return:
+        """
+        return self.get_page_child_by_type(page_id=page_id, type='page')
+
     def get_page_id(self, space, title):
         """
         Provide content id from search result by title and space
@@ -346,7 +354,8 @@ class Confluence(AtlassianRestAPI):
             if attachments['size']:
                 path = path + '/' + attachments['results'][0]['id'] + '/data'
             with open(filename, 'rb') as infile:
-                return self.post(path=path, data=data, headers=headers, files={'file': infile})
+                return self.post(path=path, data=data, headers=headers,
+                                 files={'file': (filename, infile, content_type)})
         else:
             log.warning("No 'page_id' found, not uploading attachments")
             return None
@@ -430,7 +439,8 @@ class Confluence(AtlassianRestAPI):
             log.info('Content of {page_id} differs'.format(page_id=page_id))
             return False
 
-    def update_page(self, parent_id, page_id, title, body, type='page'):
+    def update_page(self, parent_id, page_id, title, body, type='page',
+                    minor_edit=False):
         """
         Update page if already exist
         :param parent_id:
@@ -438,6 +448,8 @@ class Confluence(AtlassianRestAPI):
         :param title:
         :param body:
         :param type:
+        :param minor_edit: Indicates whether to notify watchers about changes.
+            If False then notifications will be sent.
         :return:
         """
         log.info('Updating {type} "{title}"'.format(title=title, type=type))
@@ -454,7 +466,8 @@ class Confluence(AtlassianRestAPI):
                 'body': {'storage': {
                     'value': body,
                     'representation': 'storage'}},
-                'version': {'number': version}
+                'version': {'number': version,
+                            'minorEdit': minor_edit}
             }
 
             if parent_id:
