@@ -10,9 +10,26 @@ class Jira(AtlassianRestAPI):
     def reindex_status(self):
         return self.get('rest/api/2/reindex')
 
-    def reindex(self):
-        """ Reindex the Jira instance """
-        return self.post('rest/api/2/reindex')
+    def reindex(self, comments=True, change_history=True, worklogs=True):
+        """
+        Reindex the Jira instance
+        Kicks off a reindex. Need Admin permissions to perform this reindex.
+        :param comments: Indicates that comments should also be reindexed. Not relevant for foreground reindex,
+        where comments are always reindexed.
+        :param change_history: Indicates that changeHistory should also be reindexed.
+        Not relevant for foreground reindex, where changeHistory is always reindexed.
+        :param worklogs: Indicates that changeHistory should also be reindexed.
+        Not relevant for foreground reindex, where changeHistory is always reindexed.
+        :return:
+        """
+        params = {}
+        if not comments:
+            params['indexComments'] = comments
+        if not change_history:
+            params['indexChangeHistory'] = change_history
+        if not worklogs:
+            params['indexWorklogs'] = worklogs
+        return self.post('rest/api/2/reindex', params=params)
 
     def reindex_with_type(self, indexing_type="BACKGROUND_PREFERRED"):
         """
@@ -28,6 +45,13 @@ class Jira(AtlassianRestAPI):
         """
         return self.post('rest/api/2/reindex?type={}'.format(indexing_type))
 
+    def reindex_project(self, project_key):
+        return self.post('secure/admin/IndexProject.jspa', data='confirmed=true&key={}'.format(project_key),
+                         headers=self.form_token_headers)
+
+    def reindex_issue(self, list_of_):
+        pass
+
     def jql(self, jql, fields='*all', start=0, limit=None):
         """
         Get issues from jql search result with all related fields
@@ -51,6 +75,17 @@ class Jira(AtlassianRestAPI):
             params['jql'] = jql
         return self.get('rest/api/2/search', params=params)
 
+    def csv(self, jql, limit=1000):
+        """
+        Get issues from jql search result with all related fields
+        :param jql: JQL query
+        :param limit: max results in the output file
+        :return: CSV file
+        """
+        url = 'sr/jira.issueviews:searchrequest-csv-all-fields/temp/SearchRequest.csv?tempMax={limit}&jqlQuery={jql}'.format(
+            limit=limit, jql=jql)
+        return self.get(url, not_json_response=True, header={'Accept': 'application/csv'})
+
     def user(self, username):
         return self.get('rest/api/2/user?username={0}'.format(username))
 
@@ -247,9 +282,9 @@ class Jira(AtlassianRestAPI):
         url = 'rest/api/2/project/{projectIdOrKey}/role/{roleId}'.format(projectIdOrKey=project_key,
                                                                          roleId=role_id)
         data = {}
-        if actor_type == 'group':
+        if actor_type in ['group', 'atlassian-group-role-actor']:
             data['group'] = [actor]
-        elif actor_type == 'user':
+        elif actor_type in ['user', 'atlassian-user-role-actor']:
             data['user'] = [actor]
 
         return self.post(url, data=data)
@@ -257,6 +292,31 @@ class Jira(AtlassianRestAPI):
     def issue(self, key, fields='*all'):
         return self.get('rest/api/2/issue/{0}?fields={1}'.format(key, fields))
 
+    def issue_add_json_worklog(self, key, worklog):
+        """
+
+        :param key:
+        :param worklog:
+        :return:
+        """
+        url = 'rest/api/2/issue/{}/worklog'.format(key)
+        return self.post(url, data=worklog)
+
+    def issue_worklog(self, key, started, time_sec):
+        """
+
+        :param key:
+        :param time_sec: int: second
+        :param started:
+        :return:
+        """
+        data = {
+            # "comment": "Work on {}".format(key),
+            "started": started,
+            "timeSpentSeconds": time_sec
+        }
+        return self.issue_add_json_worklog(key=key, worklog=data)
+
     def issue_field_value(self, key, field):
         issue = self.get('rest/api/2/issue/{0}?fields={1}'.format(key, field))
         return issue['fields'][field]
@@ -317,7 +377,7 @@ class Jira(AtlassianRestAPI):
         return [issue['key'] for issue in self.jql(jql)['issues']]
 
     def get_project_issues_count(self, project):
-        jql = 'project = {project}'.format(project=project)
+        jql = 'project = "{project}" '.format(project=project)
         return self.jql(jql, fields='*none')['total']
 
     def get_all_project_issues(self, project, fields='*all'):
@@ -763,9 +823,9 @@ class Jira(AtlassianRestAPI):
         url = 'rest/tempo-accounts/1/account/'
         if data is None:
             return """Please, provide data e.g.
-                       {name: "12312312321", 
-                       key: "1231231232", 
-                       lead: {name: "gonchik.tsymzhitov"},
+                       {name: "12312312321",
+                       key: "1231231232",
+                       lead: {name: "myusername"},
                        }
                        detail info: http://developer.tempo.io/doc/accounts/api/rest/latest/#-700314780
                    """
