@@ -65,13 +65,14 @@ class ServiceDesk(AtlassianRestAPI):
 
         return requests_values
 
-    def create_customer_request(self, service_desk_id, request_type_id, values_dict):
+    def create_customer_request(self, service_desk_id, request_type_id, values_dict, raise_on_behalf_of=None):
         """
         Creating customer request
 
         :param service_desk_id: str
         :param request_type_id: str
         :param values_dict: str
+        :param raise_on_behalf_of: str
         :return: New request
         """
         log.warning('Creating request...')
@@ -81,6 +82,9 @@ class ServiceDesk(AtlassianRestAPI):
             "requestFieldValues": values_dict
         }
 
+        if raise_on_behalf_of:
+            data["raiseOnBehalfOf"] = raise_on_behalf_of
+
         return self.post('rest/servicedeskapi/request', data=data)
 
     def get_customer_request_status(self, issue_id_or_key):
@@ -119,9 +123,9 @@ class ServiceDesk(AtlassianRestAPI):
         url = 'rest/servicedeskapi/request/{}/participant'.format(issue_id_or_key)
         params = {}
         if start is not None:
-            params["start"] = int(start)
+            params['start'] = int(start)
         if limit is not None:
-            params["limit"] = int(limit)
+            params['limit'] = int(limit)
 
         return self.get(url, params=params).get('values')
 
@@ -223,9 +227,9 @@ class ServiceDesk(AtlassianRestAPI):
         url_with_sd_id = 'rest/servicedeskapi/servicedesk/{}/organization'.format(service_desk_id)
         params = {}
         if start is not None:
-            params["start"] = int(start)
+            params['start'] = int(start)
         if limit is not None:
-            params["limit"] = int(limit)
+            params['limit'] = int(limit)
 
         if service_desk_id is None:
             return self.get(url_without_sd_id, headers=self.experimental_headers, params=params)
@@ -255,9 +259,9 @@ class ServiceDesk(AtlassianRestAPI):
         url = 'rest/servicedeskapi/organization/{}/user'.format(organization_id)
         params = {}
         if start is not None:
-            params["start"] = int(start)
+            params['start'] = int(start)
         if limit is not None:
-            params["limit"] = int(limit)
+            params['limit'] = int(limit)
 
         return self.get(url, headers=self.experimental_headers, params=params)
 
@@ -423,9 +427,9 @@ class ServiceDesk(AtlassianRestAPI):
         url = 'rest/servicedeskapi/request/{}/sla'.format(issue_id_or_key)
         params = {}
         if start is not None:
-            params["start"] = int(start)
+            params['start'] = int(start)
         if limit is not None:
-            params["limit"] = int(limit)
+            params['limit'] = int(limit)
 
         return self.get(url, params=params).get('values')
 
@@ -455,9 +459,9 @@ class ServiceDesk(AtlassianRestAPI):
         url = 'rest/servicedeskapi/request/{}/approval'.format(issue_id_or_key)
         params = {}
         if start is not None:
-            params["start"] = int(start)
+            params['start'] = int(start)
         if limit is not None:
-            params["limit"] = int(limit)
+            params['limit'] = int(limit)
 
         return self.get(url, headers=self.experimental_headers, params=params).get('values')
 
@@ -514,3 +518,55 @@ class ServiceDesk(AtlassianRestAPI):
         data = {'usernames': list_of_usernames}
 
         return self.post(url, headers=self.experimental_headers, data=data)
+
+    def get_queues(self, service_desk_id, include_count=False, start=0, limit=50):
+        """
+        Returns a page of queues defined inside a service desk, for a given service desk ID.
+        The returned queues will include an issue count for each queue (represented in issueCount field)
+        if the query param includeCount is set to true (defaults to false).
+
+        Permissions: The calling user must be an agent of the given service desk.
+
+        :param service_desk_id: str
+        :param include_count: bool
+        :param start: int
+        :param limit: int
+        :return: a page of queues
+        """
+        url = 'rest/servicedeskapi/servicedesk/{}/queue'.format(service_desk_id)
+        params = {}
+
+        if include_count is not None:
+            params['includeCount'] = bool(include_count)
+        if start is not None:
+            params['start'] = int(start)
+        if limit is not None:
+            params['limit'] = int(limit)
+
+        return self.get(url, headers=self.experimental_headers, params=params)
+
+    def get_issues_in_queue(self, service_desk_id, queue_id, start=0, limit=50):
+        """
+        Returns a page of issues inside a queue for a given queue ID.
+        Only fields that the queue is configured to show are returned.
+        For example, if a queue is configured to show only Description and Due Date,
+        then only those two fields are returned for each issue in the queue.
+
+        Permissions: The calling user must have permission to view the requested queue,
+        i.e. they must be an agent of the service desk that the queue belongs to.
+
+        :param service_desk_id: str
+        :param queue_id: str
+        :param start: int
+        :param limit: int
+        :return: a page of issues
+        """
+        url = 'rest/servicedeskapi/servicedesk/{0}/queue/{1}/issue'.format(service_desk_id, queue_id)
+        params = {}
+
+        if start is not None:
+            params['start'] = int(start)
+        if limit is not None:
+            params['limit'] = int(limit)
+
+        return self.get(url, headers=self.experimental_headers, params=params)
